"""
Copyright 2021 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
"""
import uuid

_align_depth = 1
_known_peers = {}
_known_downstreams = {}
_known_upstreams = {}


def prep_guid(u):
    return str(u).replace('-', '')


_own_guid = prep_guid(uuid.uuid4())

def set_align_depth(nd):
    global _align_depth
    _align_depth = nd


def get_network_align_depth():
    return _align_depth


class Server():
    def __init__(self, uid=None, start_str=''):
        if uid is None and len(start_str):
            self.uid = prep_guid(uuid.uuid4())
            while len(start_str) and not self.uid.startswith(start_str):
                self.uid = prep_guid(uuid.uuid4())
        elif uid is not None:
            self.uid = uid

        self.match_len = len(start_str)
        self.peers = {}
        self.upstreams = {}
        self.downstreams = {}

    def uid_match_len(self, other):
        # since this lovely list comp gives us matching chars in each position:
        # sum([1 for (i,j) in zip(self.uid, other.uid) if i == j else break])
        # and we just want the leading matches....
        cnt = 0
        for (i,j) in zip(self.uid, other.uid):
            if i == j: 
                cnt += 1
            else:
                break
        return cnt

    def __gt__(self, other):
        match_len = self.uid_match_len(other)
        return match_len > self.match_len

    def __lt__(self, other):
        match_len = self.uid_match_len(other)
        return match_len < self.match_len

    def __eq__(self, other):
        match_len = self.uid_match_len(other)
        return match_len == self.match_len

    def register_other(self, other):
        # Given another service instance
        # figure out our relationship
        # then, update my own datastructures
        # with that relationship. Similarly, the
        # other server will do the same.

        # the first 16 (0-f) servers will have no upstreams
        # only peers and downstreams.
        # should look into using http://www.grantjenks.com/docs/sortedcontainers/
        if self > other:
            if other.uid[:self.match_len + 1] not in self.downstreams:
                self.downstreams[other.uid[:self.match_len + 1]] = list() 
            self.downstreams[other.uid[:self.match_len + 1]].append(other)
        elif self == other:
            if other.uid[:self.match_len] not in self.peers:
                self.peers[other.uid[:self.match_len]] = list() 
            self.peers[other.uid[:self.match_len]].append(other)
        elif self < other:
            if other.uid[:self.match_len - 1] not in self.upstreams:
                self.upstreams[other.uid[:self.match_len - 1]] = list() 
            self.upstreams[other.uid[:self.match_len - 1]].append(other)

    def get_next_svr(self, guid):
        # return the server instance
        # or next best server instance to question
        key = guid[:self.match_len]

        if self.uid.startswith(key):
            # we can choose a downstream server
            # and we can also submit ourself as candidate.
            keyd = guid[:self.match_len + 1]
        elif len(key) > 1 and self.eid.startswith(key[:-1]):
            # we should pass this to a peer
            pass
        else:
            # we need to send this to an upstream server
            pass


_self_server = Server(uid=_own_guid)


def _populate_simulation(count=1024):
    global _known_downstreams, _known_peers, _known_upstreams, _own_guid, _self_server
    for i in range(0, count):
        u = prep_guid(uuid.uuid4())
        key = u[:_align_depth + 1]
        srv = None

        if u[:_align_depth + 1] == _own_guid[:_align_depth + 1]:
            if key not in _known_downstreams:
                _known_downstreams[key] = []   
            srv = Server(uid=u, start_str=u[:_align_depth + 1])
            _known_downstreams[key].append(srv)
            continue

        if u[:_align_depth] == _own_guid[:_align_depth ]:
            if key not in _known_peers:
                _known_peers[key] = []
            srv = Server(uid=u, start_str=u[:_align_depth])
            _known_peers[key].append(srv)
            continue

        if _align_depth - 1 > 0:
            if u[:_align_depth - 1] == _own_guid[:_align_depth - 1]:
                if key not in _known_upstreams:
                    _known_upstreams[key] = []
                srv = Server(uid=u, start_str=u[:_align_depth - 1])
                _known_upstreams[key].append(srv)

        if srv is not None:
            srv.register_other(_self_server)
            _self_server.register_other(srv)
        else:
            pass

def get_candidate_nodes(in_uid):
    return