"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
"""
import nord.pasion.managers.sessionmanager as sm


def test_empty_session():
    sess = sm.EconomicSession()
    assert sess.consumehw(1000.0)
    assert sess.consumehw(4.0)
    assert sess.consumestor(100000.0)
    assert sess.consumenet(-100.0)
    assert sess.spend_amount(4.2)
    sess.add_credit(99.9)
    assert sess.balance == 99.9


def test_create_session():
    sw = 'sw_addr'
    hw = 'hw_addr'
    net = 'net_addr'
    stor = 'stor_addr'
    requestor = 'Something'
    sm.create_session(requestor, sw, hw, net, stor)
    # Since the balance is NaN, but we have addresses
    # the following should return False
    assert not sm.consumehw(1000.0)
    assert not sm.consumehw(4.0)
    assert not sm.consumestor(100000.0)
    assert not sm.consumenet(-100.0)
    assert not sm.get_current_session().spend_amount(4.2)
    sm.get_current_session().add_credit(999000.0)
    assert sm.get_current_session().get_balance() == 999000.0
    assert sm.consumehw(1000.0)
    assert sm.consumehw(4.0)
    assert sm.consumestor(100000.0)
    assert sm.consumenet(-100.0)
    assert sm.get_current_session().spend_amount(4.2)
    bal = sm.get_current_session().get_balance()
    # Assumes default pricing in effect
    assert bal == 996192.8


def test_get_session():
    sm.clear_current_session()
    assert sm.get_current_session() is None
    sw = 'sw_addr'
    hw = 'hw_addr'
    net = 'net_addr'
    stor = 'stor_addr'
    requestor = 'Something'
    sm.create_session(requestor, sw, hw, net, stor)
    assert sm.get_current_session().requestor == 'Something'
