"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
Meta-class used to translate named text events into the respective single character.
"""

from nord.sigurd.utils.log import err
from nord.sigurd.utils.verboser import funlog
import nord.sigurd.handlers.keyboardhandler as keyboardhandler

_app = None


@funlog
def _bootstrap(app):
    """Due to startup sequencing, the app is passed after initialization."""
    global _app
    _app = app


@funlog
def __getattr__(name):
    """The muscle of the meta-class."""
    global _app
    if name == '__all__':
        return []
    elif name.startswith('__'):
        return globals().get(name, None)
    elif name in ('bootstrap', 'app'):
        return globals().get(f"_{name}")
    elif name.startswith('text-shift-'):
        char = name.replace('text-shift-', '').upper()
        return keyboardhandler.send_char_to_target(char)
    elif name.startswith('text-'):
        char = name.replace('text-', '').lower()
        return keyboardhandler.send_char_to_target(char)
    elif name == 'toggle':
        return keyboardhandler.toggle_target_edit()
    else:
        @funlog
        def text_input_handler():
            err("Unhandled input to text input handler: '{}'".format(name))
        return text_input_handler
