"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""
from nord.sigurd.utils.verboser import funlog
from nord.sigurd.utils.log import note, err, verb


_objects = dict()


class NordObjectMgrException(BaseException):
    pass


@funlog
def add_item(origin, app, item, mouseInteractive=True):

    if hasattr(item, 'mapentry') and hasattr(item.mapentry, 'id'):
        if has(item.mapentry.id):
            note("Tried to add already known item: {}".format(item.mapentry))
            # raise NordObjectMgrException("Tried to add already known item: {}".format(item.mapentry))
        else:
            err("Somehow Object Manager received an item "
                "to add with a mapentry.id, but not in the store (id: {})".format(item.mapentry.id))

    if mouseInteractive:
        # origin should be the point 1 unit if front of the
        # origin = origin * PLACEDIST
        # origin = cm.cwc().get_relative_point(camera, origin)
        """
            The code below is how this visual artifact is linked to
            the logical node in the graph (as stored in the Map)
        """
        # item.add_to_map()

    if hasattr(item, 'mapentry') and hasattr(item.mapentry, 'id'):
        store(item.mapentry.id, item)
    else:
        err("OBJECT MANAGER NOT STORING: {}".format(item))

    nx = origin.getX()
    ny = origin.getY()
    nz = origin.getZ()
    cx = camera.getX()  # noqa: F821
    cy = camera.getY()  # noqa: F821
    cz = camera.getZ()  # noqa: F821

    item.setPos(nx, ny, nz)

    verb(f"ObjectManager placing {item.name} at: ({nx},{ny},{nz})\nrelative to camera: ({cx},{cy},{cz})\n")


@funlog
def store(name, item):
    global _objects
    _objects[name] = item


#  @ funlog
def fetch(name):
    global _objects
    if name in _objects:
        return _objects[name]
    else:
        err("Object Manager has no object with name: {}".format(name))
        raise KeyError(name)


@funlog
def remove(name):
    global _objects
    if name in _objects:
        del _objects[name]
    else:
        err("Object Manager has no object with name: {}".format(name))
        raise KeyError(name)


@funlog
def has(name):
    global _objects
    if name in _objects:
        return True
    else:
        return False


@funlog
def _clear_for_testing():
    global _objects
    _objects.clear()
