"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""
from nord.sigurd.config.globalVars import DEFAULTPLACERSPACE,\
    DEFAULTPLACERHEIGHT,\
    DEFAULTPLACERWIDTH,\
    PLACEDIST


def behind(edge):
    """
        render the target object behind the
        source object relative to the origin
    """
    src = edge.source.visualization
    tgt = edge.target.visualization

    tgt.reparentTo(src.data)
    tgt.setPos(0, 1.2 * (src.getDepth() + tgt.getDepth()), 0)
    tgt.setIndependentMotion(False)


def infront(edge):
    """
        render the target object behind the
        source object relative to the origin
    """
    src = edge.source.visualization
    tgt = edge.target.visualization

    tgt.reparentTo(src.data)
    tgt.setPos(0, -1.2 * (src.getDepth() + tgt.getDepth()), 0)
    tgt.setIndependentMotion(False)


def left(edge):
    """
        render the target object left of the
        source object relative the origin
    """
    src = edge.source.visualization
    tgt = edge.target.visualization

    tgt.reparentTo(src.data)
    tgt.setPos(-1.2 * (src.getWidth() + tgt.getWidth()), 0, 0)
    tgt.setIndependentMotion(False)


def right(edge):
    """
        render the target object right of the
        source object relative the origin
    """
    src = edge.source.visualization
    tgt = edge.target.visualization

    tgt.reparentTo(src.data)
    tgt.setPos(1.2 * (src.getWidth() + tgt.getWidth()), 0, 0)
    tgt.setIndependentMotion(False)


def above(edge):
    """
        render the target object right of the
        source object relative the origin
    """
    src = edge.source.visualization
    tgt = edge.target.visualization

    tgt.reparentTo(src.data)
    tgt.setPos(0, 0, 1.2 * (src.getHeight() + tgt.getHeight()))
    tgt.setIndependentMotion(False)


def below(edge):
    """
        render the target object right of the
        source object relative the origin
    """
    src = edge.source.visualization
    tgt = edge.target.visualization

    tgt.reparentTo(src.data)
    tgt.setPos(0, 0, -1.2 * (src.getHeight() + tgt.getHeight()))
    tgt.setIndependentMotion(False)


def gen_dumb_placer():
    """
    This function returns a simple function which yields
    positions which can be used to distribute nodes in
    non-overlapping (hopefully) locations.
    """
    def placer():
        y = 0.0
        z = 0.0
        while True:
            for z in range(0, DEFAULTPLACERHEIGHT):
                z = z - DEFAULTPLACERHEIGHT/2.0 
                for x in range(0, DEFAULTPLACERWIDTH):
                    x = x - DEFAULTPLACERWIDTH/2.0
                    yield [x*DEFAULTPLACERSPACE, 
                        y*DEFAULTPLACERSPACE,
                        z*DEFAULTPLACERSPACE]
            y += (PLACEDIST/3)

    return placer()