"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

from nord.sigurd.utils.verboser import funlog
import nord.sigurd.managers.event as event


class SelectionManager(object):
    def __init__(s):
        s.selected = list()

    @event.callback
    @funlog
    def select(s, obj):
        if obj not in s.selected:
            s.selected.append(obj)

    @event.callback
    @funlog
    def deselect(s, obj):
        if obj in s.selected:
            s.selected.remove(obj)

    @funlog
    def apply(s, func):
        for i in s.selected:
            func(i)


_current_sm = SelectionManager()
_sm_stack = []


def select(obj):
    global _current_sm
    _current_sm.select(obj)


def deselect(obj):
    global _current_sm
    _current_sm.deselect(obj)


def apply(func):
    global _current_sm
    _current_sm.apply(func)


def push():
    global _current_sm, _sm_stack
    _sm_stack.append(_current_sm)
    _current_sm = SelectionManager()


def pop():
    global _sm_stack, _current_sm
    _current_sm = _sm_stack.pop()
