"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
Base module for the source and target ends of a connection.
"""

from nord.sigurd.utils.verboser import funlog
from nord.sigurd.shell.model import Model


class ConnEnd(Model):
    """Inherited by Source and Target ends for common functionality."""

    @funlog
    def __init__(self, uid, model, app, tag, parent, connection, space="nord"):
        """Call the Models constructor and establish end values."""
        super().__init__(uid, model, app, tag, space=space)
        self.linkedTo = parent
        self.connection = connection
        self.x = parent.getX()
        self.y = parent.getY()
        self.z = parent.getZ()

    @funlog
    def link(self, parent):
        """Establish connection to parent object."""
        self.linkedTo = parent

    @funlog
    def setPos(self, x, y, z):
        """
        Set the position of this end, and trigger whole connection recalc.

        The passed position is expected to be the center of the connected
        node. So, this end needs to figure out the vector between the
        itself and the other end as well as the needed offset along said
        vector. Once it compute'self its position, it needs to tell the
        other components in the connection to update their position as well.

        This connection implementation is one for direct linear connections
        between nodes.

        """
        if self.data.hasParent():
            self.x = x
            self.y = y
            self.z = z
            self.connection.calcPos()

    @funlog
    def reparentTo(self, node):
        """Implement the reparent function for rendering purposes."""
        if self.data.getParent() != node and self.data.has_parent():
            self.data.reparentTo(node)
            self.connection.reparentTo(node)

    @funlog
    def handle_clicked(self, origin, app):
        """Pass clicks to the parent connection for handling."""
        self.connection.handle_clicked(origin, app)
