"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
The node managing the visualization of containers.
"""

from nord.sigurd.utils.verboser import funlog
import nord.nord.utils.SpaceManager as use
import nord.sigurd.managers.containermanager as cm
import nord.sigurd.shell.vismap as vm
import nord.sigurd.managers.objectmanager as om


@funlog
def draw(edge):
    """The visual precedence function to render container's correctly."""
    # if not hasattr(edge.source, "visualization") or not hasattr(edge.target, "visualization"):
    #     return
    src = om.fetch(edge.source.id)
    tgt = om.fetch(edge.target.id)

    cm.add_if_missing(src.mapentry.id)
    cm.enter(src.mapentry.id)
    cm.add_to_current(tgt)
    cm.exit()


@funlog
def contains(edge):
    """
    Create a Contains Edge given the input edge.

    This method is given a source and target visualization
    and needs to return a Contains(Edge) to the caller
    """
    src = edge.source
    target = edge.target
    old_space = use.current_space()
    use.space("nord")
    contains = use.module("nord.nord.edges.Contains")
    rv = contains(vm.get_working_map())
    use.space(old_space)
    rv.set_nodes(src, target, "contains")
    return rv
