"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

from nord.sigurd.shell.model import Model
from nord.sigurd.utils.verboser import funlog
from nord.sigurd.visibles.ui.text import Text, Anchor
import nord.sigurd.managers.event as event


class LabeledModel(Model):
    _one_line = True

    @funlog
    def __init__(s, uid, name, app, tag='clickable', space="nord"):
        super().__init__(uid, name, app, tag, space=space)
        if hasattr(s, "mapentry"):
            def_label = s.mapentry.name
        else:
            def_label = name + ' ' + uid
        s.label = Text(f"Label_{uid}", app,
                       parent=s.data,
                       default_contents=def_label,
                       initscale=0.5,
                       anchor_h=Anchor.CENTER,
                       anchor_v=Anchor.BOTTOM,
                       one_line=s._one_line)
        s.label.setPos(-1.0 * s.label.getWidth()/6,
                       0,
                       s.getHeight() * 0.55 + s.label.getHeight()/4)

        s.label.addCallback('on_change', s.applyLabelChange)
        s.label_changed = False

        @event.callback
        @funlog
        def on_map_update_name():
            if hasattr(s, "mapentry"):
                label = s.mapentry.name
            else:
                label = name + ' ' + uid
            s.setLabelText(label)
        s.onMappedFuncs.append(on_map_update_name)

    @event.callback
    @funlog
    def applyLabelChange(s, newText):
        s.name = newText
        # Here we'll apply the name from the UI over the generated name to the Map name, if there is one
        if hasattr(s, 'mapentry') and hasattr(s.mapentry, 'name'):
            s.mapentry.name = s.name

    @funlog
    def setLabelText(self, text):
        self.label.setText(text)
        self.label_changed = True

    @funlog
    def is_default_label(self):
        """Return True if the label has not changed from default."""
        return not self.label_changed

    @funlog
    def add_label_change_callback(self, func):
        """Add the provided function to the label's callback list."""
        self.label.addCallback('on_change', func)
