"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
The MainMenu for the visual system.

This menu is used as the initial point of
interaction with a map.
"""


import sys
from nord.sigurd.shell.menu import Menu
from nord.sigurd.shell.menuItem import MenuItem
from nord.sigurd.shell.submenu import SubMenu
from nord.sigurd.utils.log import err, warn
from nord.sigurd.utils.verboser import funlog
import nord.sigurd.managers.statemanager as em
import nord.nord.utils.SpaceManager as use
import nord.sigurd.managers.event as event


class MainMenu(Menu):
    """Eponymous class for this module."""

    def __init__(self, space="nord"):
        """Extend the base menu censtructor."""
        super().__init__(space)
        self.space_menus = {}

    def __repr__(self):
        """Return a string of this menu."""
        return "MAIN {}".format(super().__repr__())

    @funlog
    def add_space_menu(self, space, menu, titles={}):
        """For all nodes in the given space, populate submenu and add if missing."""
        if space not in self.space_menus:
            self.space_menus[space] = SubMenu(f"menusubaddnodes{space}",
                                              self,
                                              space=space,
                                              title=titles.get(space, None))
            # to be polite, save and then restore the current space
            ospace = use.current_space()
            use.space(space)
            has_menuconf = False
            if use.file_exists('config/menu.conf'):
                # This means the space (extension) has its own ideas as to
                # how it's menu should be setup. So, process it's menu config
                # instead of automatically populating the nodes menu.
                has_menuconf = True
            use.space(ospace)

            if not has_menuconf:
                self.populate_nodes_menu(self.space_menus[space], space=space)
            else:
                self.space_menus[space].load_from_file('config/menu.conf', space)
            menu.add_item(self.space_menus[space])

    @funlog
    def load(self, titles={}):
        """Populate the menu with its items."""
        spaces_sub = SubMenu("menusubaddnodesnord", self, title="Items to add")
        for space in use.get_spaces():
            self.add_space_menu(space, spaces_sub, titles=titles)
        self.add_item(spaces_sub)

        @event.callback
        @funlog
        def action_cbk(*args, **kwargs):
            err("Action Submenu Selected")

        @event.callback
        @funlog
        def run_inspect(*args, **kwargs):
            import nord.sigurd.managers.containermanager as cm
            import nord.sigurd.shell.vismap as vm
            print("\n\n---------------vvvvvv--------------------\n")
            cm.cwc().ls()
            print("\n-----------------^^^^^^------------------\n\n")
            print("\n\n\n------------- working map ------------\n\n")
            print(vm.get_working_map())
            print("\n\n------------- end working map ------------\n\n")

        @event.callback
        @funlog
        def do_quit(*args, **kwargs):
            sys.exit(0)

        # self.add_item(MenuItem('action', em.add_action))
        # self.add_item(MenuItem('data', em.add_data))
        # self.add_item(MenuItem('load', em.load))
        self.add_item(MenuItem('action', run_inspect, title="Dump state to console"))
        self.add_item(MenuItem('data', em.add_space_to_menu, title="Link Extension"))
        self.add_item(MenuItem('anomaly', do_quit, title="Quit"))
        # self.add_item(MenuItem('container', em.add_container))
        # self.add_item(MenuItem('save', em.save))
        self.add_item(MenuItem('exitcontainer', em.exit_container, title="Exit Container"))

    @funlog
    def check_selection(self, *args, **kwargs):
        """Override the base class check_selection."""
        warn("Main Menu check_selection called, but it shouldn't have been")

    @event.callback
    @funlog
    def show(self, origin, app):
        """Not sure why this is here."""
        # origin = ei.args[0]
        # app = ei.args[1]
        super().show(origin, app)

    @staticmethod
    @funlog
    def internal_reinit_module():
        mm = MainMenu()
        sys.modules[__name__] = mm


mm = MainMenu()
sys.modules[__name__] = mm
