"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
The modelmenu module is the generic class that creates menus on models.
"""

from nord.sigurd.shell.menu import Menu
from nord.sigurd.utils.log import warn, err, info
from nord.sigurd.utils.verboser import funlog
import nord.sigurd.managers.statemanager as em
import nord.sigurd.shell.application as app
import nord.sigurd.managers.event as event
import nord.sigurd.managers.containermanager as cm


@event.callback
@funlog
def hide_model_menu(*args, **kwargs):
    """Called by the eventmanager."""
    model = args[0]
    model.hide()


@funlog
def check_selection(*args, **kwargs):
    """Probably of negligible value."""
    warn("Model Menu click detected, but no object clicked...")


class ModelMenu(Menu):
    """
    Main export of this module.

    Used to as the basis for model specific menus.
    """

    @funlog
    def check_selection(self, origin):
        """
        Detect which item in the menu was selected.

        Uses the global "base" injected by the panda3d modules
        imported elsewhere.
        """
        mpos = base.mouseWatcherNode.getMouse()  # noqa: F821
        self.pickerRay.setFromLens(base.camNode, mpos.getX(), mpos.getY())  # noqa: F821
        self.pointerTraverser.traverse(cm.cwc())
        if self.pointerCollisionHandler.getNumEntries() > 0:
            self.pointerCollisionHandler.sortEntries()
            p_obj = self.pointerCollisionHandler.getEntry(0).getIntoNodePath()
            p_obj = p_obj.findNetTag('menuselectable')
            if not p_obj.isEmpty():
                msg = "Menu Selected Object: {}" \
                      .format(p_obj.getTag('menuselectable'))
                info(msg)
                obj = self.itemsByName[p_obj.getTag('menuselectable')]
                obj.activate(self.pickerRay.getOrigin(), app)
                self.hide(event.EventiInstance(self))

    @funlog
    def show(self, origin, app):
        """
        Relocate and show the model's menu.

        also turns the menu to face the camera.
        """
        warn("Showing Model Menu '{}'".format(self.menuName))

        nx = origin.getX()
        ny = origin.getY()
        nz = origin.getZ()

        self.data.setPos(nx, ny, nz)

        self.data.setHpr(camera.getHpr())  # noqa: F821
        # for itm in s.items:
        #   itm.data.lookAt(camera)

        @funlog
        def callback():
            err("The menu should be hidden after this...")
            self.hide(event.EventInstance(self.__class__.__name__))
        em.listen('hide_model_menu', 'modelMenu', callback)

        self.data.show()
        self.visible = True

    @event.callback
    @funlog
    def hide(self):
        """Hide the visible menu."""
        em.unlisten('hide_model_menu', 'modelMenu')
        super().hide(event.EventInstance(super().__class__.__name__))
