"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

from panda3d.core import Fog, NodePath, LineSegs, PointLight, VBase4
from nord.sigurd.utils.verboser import funlog  # noqa: F401
from nord.sigurd.utils.log import info

FOG_DENSITY = 0.003


def getWorldPos(node):
    """
    given a node, find it's position in the world
    by traversing up it's ancestry to the topmost
    node.
    """
    import nord.sigurd.managers.containermanager as cm
    while not node.getParent() == cm.cwc() and\
            not node.getParent().isEmpty() and\
            not repr(node.getParent()).startswith('__nested_render__'):
        info("Type: {} Node: {} parent: {}"
             .format(type(node), repr(node), repr(node.getParent())))
        node = node.getParent()

    return node.getPos()


class WorldView(object):

    # Load the environment model.
    # self.scene = self.loader.load_model("models/environment")
    # Reparent the model to render.
    # self.scene.reparentTo(self.render)
    # Apply scale and position transforms on the model.
    # self.scene.setScale(0.25, 0.25, 0.25)
    # self.scene.setPos(-8, 42, 0)

    @classmethod
    @funlog
    def set_lighting(s, render):
        plight = PointLight('plight')
        plight.setColor(VBase4(0.2, 0.2, 0.2, 1))
        plnp = render.attachNewNode(plight)
        plnp.setPos(120, 120, 120)
        render.setLight(plnp)

        plight = PointLight('plight')
        plight.setColor(VBase4(1, 1, 1, 1))
        plnp = render.attachNewNode(plight)
        plnp.setPos(120, -120, 120)
        render.setLight(plnp)

        plight = PointLight('plight')
        plight.setColor(VBase4(0.2, 0.2, 0.2, 1))
        plnp = render.attachNewNode(plight)
        plnp.setPos(-120, 120, 120)
        render.setLight(plnp)

        plight = PointLight('plight')
        plight.setColor(VBase4(1, 1, 1, 1))
        plnp = render.attachNewNode(plight)
        plnp.setPos(-120, -120, 120)
        render.setLight(plnp)

    @classmethod
    @funlog
    def set_fog(s, render, base):
        colour = (0.1, 0.1, 0.1)
        expfog = Fog("Scene-wide exponential Fog object")
        expfog.setColor(*colour)
        expfog.setExpDensity(FOG_DENSITY)
        render.setFog(expfog)
        base.setBackgroundColor(*colour)

    @classmethod
    @funlog
    def draw_grid(s, render):
        extent = 300
        space = 50
        for x in range(0, extent, space):
            for y in range(0, extent, space):
                for z in range(0, extent, space):
                    for a in [1, -1]:
                        for b in [1, -1]:
                            if x <= y and y <= z:
                                ls = LineSegs("XZ{}{}".format(x, z))
                                ls.drawTo(a*x, -1*extent, b*z)
                                ls.drawTo(a*x, extent, b*z)
                                node = ls.create()
                                np = NodePath(node)
                                np.reparentTo(render)
                                np.setColor(0.5, 0.5, 0.5, 0.02, 1)

                                ls = LineSegs("YZ{}{}".format(y, z))
                                ls.drawTo(-1*extent, a*y, b*z)
                                ls.drawTo(extent, a*y, b*z)
                                node = ls.create()
                                np = NodePath(node)
                                np.reparentTo(render)
                                np.setColor(0.5, 0.5, 0.5, 0.02, 1)

                                ls = LineSegs("XY{}{}".format(x, y))
                                ls.drawTo(a*x, b*y, -1*extent)
                                ls.drawTo(a*x, b*y, extent)
                                node = ls.create()
                                np = NodePath(node)
                                np.reparentTo(render)
                                np.setColor(0.5, 0.5, 0.5, 0.02, 1)
