"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

from nord.sigurd.utils.verboser import funlog
from nord.sigurd.shell.visnode import VisNode
import nord.sigurd.managers.event as event


class Static(VisNode):
    def __init__(self, uid, app, name='static', space="nord"):
        super().__init__(uid, name, app, space=space)

    @event.callback
    @funlog
    def applyLabelChange(self, newText):
        self.name = newText
        # Here we'll apply the name from the UI over the
        # generated name to the Map name, if there is one
        if hasattr(self, 'mapentry'):
            if hasattr(self.mapentry, 'name'):
                self.mapentry.name = self.name
            if not self.mapentry.is_set_property('value'):
                self.mapentry.set_property("value", newText)

    @funlog
    def handlePropertyChange(self, property, text):
        """
        If the value property had never been set, reset the label to show the new value.

        But, only change the label if the label is still the default.
        """
        if property.name == 'value' and\
                not self.mapentry.is_set_property('value') and\
                self.is_default_label():
            self.setLabelText(text)

    @funlog
    def update_tooltip(self, tooltip):
        """Update the contents of this model's tooltip."""
        value = "<NO VALUE>"
        if hasattr(self, 'mapentry'):
            if self.mapentry.is_set_property('value'):
                value = f"""'{self.mapentry.get_property("value")}'"""
        tooltip.set_text(f"Static: {value}")
