"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
Text module manages in-world text.
"""

from nord.sigurd.utils.verboser import funlog
import nord.sigurd.managers.statemanager as em
import nord.sigurd.managers.containermanager as cm
from nord.sigurd.managers.callbackmanager import CallbackManager
from panda3d.core import TextNode, CardMaker, NodePath, DecalEffect
from nord.sigurd.visibles.ui.text import Anchor, HAlign
import nord.sigurd.managers.objectmanager as om


class Button(CallbackManager):
    def __init__(
        self,
        uid,
        parent=None,
        text="Text",
        initscale=1.0,
        command=None,
        anchor_h=Anchor.LEFT,
        anchor_v=Anchor.TOP,
        align_h=HAlign.LEFT
    ):
        """Initialize the text instance."""
        CallbackManager.__init__(self)
        if parent is None:
            parent = cm.cwc()
        self.parent = parent
        self.data = None
        self.uid = uid
        self.command = command
        # self.text_np = parent.attachNewNode(tnp)
        self.uid = uid
        self.name = uid
        self.contents = text
        self.line_pos = 0
        self.tail = ""
        self.anchor_h = anchor_h
        self.anchor_v = anchor_v
        self.align_h = HAlign
        self.anchor_pos = None
        self.addEvent("on_pressed")
        om.store(f"{uid}", self)
        self.draw()

    @funlog
    def addCallback(self, name, func):
        pass

    @funlog
    def getTightBounds(self):  # noqa: N802
        """Return the bounding box."""
        return self.text_np.getTightBounds()

    @funlog
    def getString(self):  # noqa: N802
        """Return the text string managed by this instance."""
        return self.contents + self.tail

    @funlog
    def clear(self):
        """Remove contents of this text area."""
        self.contents = ""
        self.tail = ""
        self.draw()

    @funlog
    def centerHorizontally(self):
        """Center the text horizontally, but not vertically."""
        self.center(cent_horiz=True, cent_vert=False)

    @funlog
    def center(self, cent_horiz, cent_vert):
        """
        Center the text according to flags cent_horiz and cent_vert.

        calculate the size of each row of text
        and re-position the glyphs on both sides of zero X
        """
        pass

    @funlog
    def draw(self):
        """
        Method to trigger the rendering of the glyphs.

        Render the contents as glyphs
        parented by self.data
        """
        uid = self.uid
        parent = self.parent
        oldPos = None
        if self.data is not None:
            oldPos = self.text_np.getPos()
            self.text_np.removeNode()
        self.data = TextNode(uid)
        cm = CardMaker(f"cm--{uid}")
        self.data.setText(self.contents)
        cm.setFrame(self.data.getFrameActual())
        tgm = cm.generate()
        self.card = NodePath(tgm)
        self.card.setColor(0.4, 0.4, 0.4, 0.6)
        self.data.setTextColor(1.0, 1.0, 1.0, 1.0)
        tnp = self.card.attachNewNode(self.data)
        self.text_np = parent.attachNewNode(tgm)
        self.card.setTag("clickable", uid)
        self.card.setEffect(DecalEffect.make())
        self.cardMaker = cm
        self.text_np.setScale(0.33)
        if oldPos is not None:
            self.text_np.setPos(oldPos)

        self.text_np.show()


    @funlog
    def setPos(self, x, y, z):  # noqa: N802
        """Expose the internal set position method."""
        self.text_np.setPos(x, y, z)

    @funlog
    def getWidth(self):
        """Return the width of the text"""
        return self.data.getWidth()

    @funlog
    def getHeight(self):
        """Return the height of the text"""
        return self.data.getHeight()

    @funlog
    def handle_clicked(self, *args, **kwargs):
        """Handle a click on any of the visible text."""
        # em.post_pending('hide_model_menu', self.menu)
        # em.post_pending('model_toggle_selected', self)
        self.command(*args, **kwargs)
