"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

import nord.sigurd.utils.namegen as ng
from panda3d.core import TextNode

class Label(object):
    """Class to render 2d text on screen."""

    def __init__(self, text, pos, parent):
        """Initialize the label with text and position."""
        self.text = text
        self.pos = pos
        self.parent = parent
        uid = ng.id()
        self.data = TextNode(uid)
        self.data.setText(self.text)
        self.data.setTextColor(1.0, 1.0, 1.0, 1.0)
        self.text_np = self.parent.attachNewNode(self.data)
        self.text_np.setScale(0.33)
        self.text_np.setPos(self.pos)

    def show(self):
        """Render the label."""

        self.text_np.show()

    def hide(self):
        """Remove the label from the screen."""
        self.text_np.removeNode()

    def getWidth(self):
        """Return the width of the label."""
        return self.data.getWidth()
