"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""
from nord.sigurd.managers.callbackmanager import CallbackManager


def test_add_event():
    cbm = CallbackManager()
    cbm.addEvent('button_pressed')

    assert cbm.eventSubscribers['button_pressed'] == []


def test_add_callback():
    cbm = CallbackManager()

    def cbk(*args, **kwargs):
        return None
    cbm.addCallback('button_pressed', cbk)
    assert not cbm.eventSubscribers
    cbm.addEvent('button_pressed')
    cbm.addCallback('button_pressed', cbk)
    assert len(cbm.eventSubscribers['button_pressed']) == 1
    cbm.addCallback('button_pressed', cbk)
    assert len(cbm.eventSubscribers['button_pressed']) == 1


def test_trigger_event():
    cbm = CallbackManager()
    d = {"called": 0}\

    def gen_cbk(obj):

        def cbk(*args, **kwargs):
            obj['called'] += 1
            return None
        return cbk

    cbm.addEvent('button_pressed')
    cbm.addCallback('button_pressed', gen_cbk(d))
    cbm.triggerEvent('button_pressed')
    assert d['called'] == 1
