"""
Copyright 2020 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
"""
import nord.sigurd.managers.containermanager as CM


def test_prepare(init_panda3d):
    CM.clear()
    CM.prepare()
    _render = CM._containers['__top_of_the_world__'][0]
    _world = CM._containers['__top_of_the_world__'][1]
    _render2d = CM._containers['__top_of_the_world__'][2]
    # _placeholder = CM._containers['__top_of_the_world__'][3]
    # _placer = CM._containers['__top_of_the_world__'][4]
    assert len(CM._containers.keys()) == 1
    assert len(CM._containers['__top_of_the_world__']) == 5
    assert CM._current_name == '__top_of_the_world__'
    assert CM._current[0] is _render
    assert CM._current[1] is _world
    assert CM._current[2] is _render2d


def test_add(init_panda3d):
    CM.add('TEST_CONTAINER')
    assert "TEST_CONTAINER" in CM._containers
    np = CM._containers["TEST_CONTAINER"][0]
    assert np.name == '__nested_render__TEST_CONTAINER'


def test_add_if_missing(init_panda3d):
    assert "TEST_CONTAINER2" not in CM._containers
    CM.add_if_missing("TEST_CONTAINER2")
    assert "TEST_CONTAINER2" in CM._containers
    np = CM._containers["TEST_CONTAINER2"][0]
    assert np.name == '__nested_render__TEST_CONTAINER2'


def test_has(init_panda3d):
    assert not CM.has("TEST_CONTAINER3")
    CM.add_if_missing("TEST_CONTAINER3")
    assert "TEST_CONTAINER3" in CM._containers
    np = CM._containers["TEST_CONTAINER3"][0]
    assert np.name == '__nested_render__TEST_CONTAINER3'
    assert CM.has("TEST_CONTAINER3")


def test_current_name(init_panda3d):
    CM.clear()
    assert CM._current_name is None
    assert CM.get_current_name() is None
    CM.prepare()
    assert CM.get_current_name() == '__top_of_the_world__'


def test_enter_and_exit(init_panda3d):
    CM.clear()
    CM.add_if_missing("TEST_CONTAINER")
    CM.add_if_missing("TEST_CONTAINER2")
    CM.add_if_missing("TEST_CONTAINER3")
    assert CM.has("TEST_CONTAINER")
    assert CM.has("TEST_CONTAINER2")
    assert CM.has("TEST_CONTAINER3")
    CM.enter("TEST_CONTAINER")
    assert CM.get_current_name() == "TEST_CONTAINER"
    CM.enter("TEST_CONTAINER3")
    assert CM.get_current_name() == "TEST_CONTAINER3"
    CM.enter("TEST_CONTAINER2")
    assert CM.get_current_name() == "TEST_CONTAINER2"
    assert len(CM._stack) == 4
    CM.exit()
    assert CM.get_current_name() == "TEST_CONTAINER3"
    assert len(CM._stack) == 3
    CM.exit()
    assert CM.get_current_name() == "TEST_CONTAINER"
    assert len(CM._stack) == 2
    CM.exit()
    assert CM.get_current_name() == "__top_of_the_world__"
    assert len(CM._stack) == 1


def test_is_top(init_panda3d):
    CM.clear()
    assert CM.is_top()
    CM.add_if_missing("TEST_CONTAINER4")
    assert CM.has("TEST_CONTAINER4")
    CM.enter("TEST_CONTAINER4")
    assert not CM.is_top()
    CM.exit()
    assert CM.is_top()


def test_next_loc(init_panda3d):
    CM.prepare()
    a = CM.next_loc()
    assert a == [-10.0, 0.0, -10.0]
    for i in range(0, 45):
        a = CM.next_loc()
    assert a == [-10.0, 66.66666666666667, 6.0]
    for i in range(0, 446):
        a = CM.next_loc()
    assert a == [-6.0, 1266.6666666666667, 2.0]

    CM.add("billy")
    CM.enter("billy")
    a = CM.next_loc()
    assert a == [-10.0, 0.0, -10.0]
    for i in range(0, 45):
        a = CM.next_loc()
    assert a == [-10.0, 66.66666666666667, 6.0]
    for i in range(0, 446):
        a = CM.next_loc()
    assert a == [-6.0, 1266.6666666666667, 2.0]
    CM.exit()
    a = CM.next_loc()
    a = CM.next_loc()
    a = CM.next_loc()
    assert a == [6.0, 1266.6666666666667, 2.0]
    CM.enter("billy")
    a = CM.next_loc()
    assert a == [-2.0, 1266.6666666666667, 2.0]
