"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""
import nord.sigurd.managers.objectmanager as om
# from mock import Mock, patch

import pytest
from panda3d.core import Point3
import nord.sigurd.shell.vismap as vm


def test_store_remove():
    om._clear_for_testing()
    tmp = {"a": 1}
    om.store("tmp", tmp)
    assert om._objects["tmp"] == tmp
    om.remove("tmp")
    assert om._objects == {}


def test_fetch():
    om._clear_for_testing()
    tmp = {"a": 1}
    om.store("tmp", tmp)
    other = om.fetch("tmp")
    assert other == tmp
    om.remove("tmp")


def test_has():
    om._clear_for_testing()
    tmp = {"a": 1}
    om.store("tmp", tmp)
    assert om.has("tmp")
    om.remove("tmp")
    assert not om.has("tmp")


@pytest.mark.panda3d()
def test_objectmgr_from_vismap(init_panda3d):
    """
    Test that visual map changes affect the contents of the object manager
    as expected.

    That is to say, adding nodes and edges create all the requisite entries
    AND, removing them cleans out all the appropriate contents as well.

    This should also include any generated menu contents as well.
    """
    karte = vm.get_working_map()
    vm.create_node("Data", Point3(-1.0, 0.0, 0.0), space="nord")
    data = karte.nodes[list(karte.nodes.keys())[0]]
    vm.create_node("Static", Point3(1.0, 0.0, 0.0), space="nord")
    static = karte.nodes[list(karte.nodes.keys())[1]]
    vm.create_edge(static, data, 'assign', None, None)

    karte.ship_changes()

    # visData = om.fetch(data.id)
    # visStatic = om.fetch(static.id)
    # visAssign = om.fetch(f"{static.id}:{data.id}:assign")

    did = data.id
    karte.rem_node(data.id)

    # Now, not only should visData no longer exist
    # in the object manager store, but, visAssign
    # shouldn't either
    caught_exc = None
    try:
        om.fetch(f"{static.id}:{data.id}:assign")
    except KeyError as e:
        caught_exc = e

    assert caught_exc is not None

    caught_exc = None
    try:
        om.fetch(did)
    except KeyError as e:
        caught_exc = e

    assert caught_exc is not None

    stid = static.id
    static.delete()
    karte.ship_changes()

    caught_exc = None
    try:
        om.fetch(stid)
    except KeyError as e:
        caught_exc = e

    assert caught_exc is not None
