"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

import nord.sigurd.managers.statemanager as sm
import nord.sigurd.managers.eventmap as em
# from mock import Mock, patch


def test_freespace_click_in_default():
    "Start in the current state, with no handlers loaded."
    em.clear()
    sm.internal_initialize_statemanager()
    intercepts, current, states, listeners = sm.test_use_only_get_internal_vars()
    sm.mouse_freespace_click(None, None)
    intercepts, current, states, listeners = sm.test_use_only_get_internal_vars()
    assert current == 'state_main_menu_visible'


def test_open_close_main_menu_freespace():
    """
    Create and patch into eventmap the hide_main_menu
    function to ensure that it is injected by the state machine
    and executed when expected.
    """

    # Clear the eventmap state to prevent pollution from prior tests
    em.clear()

    # Set up mock function to be injected
    # into the event map
    call_state = {}
    def gen_hide_main_menu(outer_state):

        def hide_main_menu(*args, **kwargs):
            outer_state["hide_called"] = True
        return hide_main_menu

    # Place the dummy function into the eventmap
    em.test_use_only_inject_handler('hide_main_menu', gen_hide_main_menu(call_state))

    # prevent any possible prior test pollution from affecting the statemanager
    sm.internal_initialize_statemanager()

    # for debug purposes, grab the internals from the state manager
    # intercepts, current, states, listeners = sm.test_use_only_get_internal_vars()
    sm.mouse_freespace_click(None, None)
    sm.mouse_freespace_click(None, None)
    # Now, grab the internals to evaluate that the
    # state machine is in the correct state
    intercepts, current, states, listeners = sm.test_use_only_get_internal_vars()
    assert current == 'default'
    # And, lastly, verify that the function we mocked was called.
    assert "hide_called" in call_state


def test_open_close_main_menu_object_clicks():
    """
    Create and patch into eventmap the hide_main_menu
    function to ensure that it is injected by the state machine
    and executed when expected.
    """

    # Clear the eventmap state to prevent pollution from prior tests
    em.clear()

    # Set up mock function to be injected
    # into the event map
    call_state = {'clicks': 0}
    def gen_hide_main_menu(outer_state):

        def hide_main_menu(*args, **kwargs):
            outer_state["hide_called"] = True
        return hide_main_menu

    def gen_clickable_object_clicked(outer_state):

        def clickable_object_clicked(*args, **kwargs):
            outer_state["clicks"] += 1
        return clickable_object_clicked

    # Place the dummy function into the eventmap
    em.test_use_only_inject_handler('hide_main_menu', gen_hide_main_menu(call_state))
    em.test_use_only_inject_handler('clickable_object_clicked', gen_clickable_object_clicked(call_state))

    # prevent any possible prior test pollution from affecting the statemanager
    sm.internal_initialize_statemanager()

    # Open the menu, click on two objects, then click out of the menu
    sm.mouse_freespace_click()
    intercepts, current, states, listeners = sm.test_use_only_get_internal_vars()
    sm.clickable_object_clicked()
    intercepts, current, states, listeners = sm.test_use_only_get_internal_vars()
    assert call_state["clicks"] == 1
    assert "hide_called" in call_state
    sm.clickable_object_clicked()
    assert call_state["clicks"] == 2
    intercepts, current, states, listeners = sm.test_use_only_get_internal_vars()
    assert current == 'default'
