"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""
# import nord.sigurd.managers.connectionmanager as cm
import nord.nord.nodes.Static as Static
import nord.sigurd.managers.eventmap as em
import nord.sigurd.managers.event as event
import nord.nord.Map as Map
import pytest
from panda3d.core import Point3
from nord.sigurd.visibles.static import Static as vStatic
from nord.sigurd.visibles.data import Data as vData
import nord.nord.Edge as Edge


def test_start_conn():
    source = Static(Map(None))
    ctype = 'assign'
    # First, we'll need an event mapping to the method we want to call
    # this permits making configurable behaviors.
    em.proc_line("test_start_conn:nord.sigurd.managers.connectionmanager.start_conn")
    # Now, call the event, which will call the start_conn function
    # though the __getatrr__ magic in eventmap
    em.test_start_conn(event.EventInstance("", source, ctype))
    # Now check that the variables are where we expect them.
    # the import is here, in function, rather than the top
    # having at the top was preventing vscode's ability to
    # discover and debug the tests!
    # Note, now that it is imported, all subsequent uses
    # of the connection manager, will hit the same instance.
    import nord.sigurd.managers.connectionmanager as cm
    assert cm.pendingSource == source
    assert cm.pendingCtype == ctype


def make_model_mock(space="nord"):
    from mock import Mock, MagicMock

    def gtb():
        return (Point3(0, 0, 0), Point3(1, 0, 0))

    def mockd_actor_init():
        self = Mock()
        self.getTightBounds = gtb
        self.getHeight = lambda: 1.0
        self.getWidth = lambda: 1.0
        self.getPos = lambda: Point3(0, 0, 0)
        self.getColorScale = lambda: (0, 0, 0)
        self.reparentTo = lambda *args: None
        self.menuName = "OTHER TEST MENUNAME"
        self.ModelMenu = Mock()
        self.ModelMenu.menuName = 'TEST MODELMENUNAME'
        return self

    def mockd_model_init(*args, **kwargs):
        args[0].uid = '123'
        args[0].parent = Mock()
        args[0].data = Mock()
        args[0].model = mockd_actor_init()
        args[0].label = Mock()
        args[0].itemsByName = {}
        args[0].name = 'bob'
        args[0].add_item = lambda *args: None
        args[0].items = []
        args[0].tooltip = Mock()
        args[0].back = Mock()
        args[0].onMappedFuncs = []
        args[0].onUnmappedFuncs = []
        args[0].links = {}
        args[0].submenus = []
        args[0].ModelMenu = Mock()
        args[0].ModelMenu.menuName = 'TEST MODELMENUNAME'
        args[0].menuName = 'TEST MODELMENUNAME'

        args[0].getTightBounds = gtb
        # args[0].model.getTightBounds = gtb
        args[0].back.getTightBounds = gtb
        # args[0].model.create_connection_menu = lambda: None
        args[0].create_connection_menu = lambda *args: None
        args[0].getHeight = lambda: 1.0
        args[0].label.getHeight = lambda: 1.0
        args[0].label.getWidth = lambda: 1.0
        args[0].data.getPos = lambda: Point3(0, 0, 0)
        args[0].data.getColorScale = lambda: (0, 0, 0)
        args[0].reparentTo = lambda *args: None
        args[0].space = space
        import nord.sigurd.shell.model as m
        try:
            m.set_find_mapentry(args[0])
        except Exception:
            pass
        return None

    import nord.sigurd.shell.model as m
    m.Model.__init__ = mockd_model_init

    import nord.sigurd.shell.menu as u
    u.Menu.__init__ = mockd_model_init

    import nord.sigurd.managers.modelmanager as mm
    mm.load_model = lambda *args: MagicMock()


@pytest.mark.skip(reason="INCOMPLETE TEST")
def test_complete_conn():
    from mock import Mock, patch

    make_model_mock()

    @patch('builtins.render', Mock(), create=True)
    @patch('builtins.render2d', Mock(), create=True)
    @patch('builtins.camera', Mock(), create=True)
    def start_connection():
        source = vStatic('1', Mock())
        ctype = 'assign'
        em.proc_line("test_start_conn:nord.sigurd.managers.connectionmanager.start_conn")
        em.test_start_conn(event.EventInstance("", source, ctype))

    start_connection()
    import nord.sigurd.managers.connectionmanager as cm
    assert cm.pendingSource is not None
    assert cm.pendingCtype is not None

    @patch('builtins.render', Mock(), create=True)
    @patch('builtins.render2d', Mock(), create=True)
    @patch('builtins.camera', Mock(), create=True)
    def complete_connection():
        target = vData('2', Mock())
        em.proc_line("test_complete_conn:nord.sigurd.managers.connectionmanager.complete_conn")
        args = (target, None)
        em.test_complete_conn(*args)
        edge = target.mapentry.get_sources('assign')[0]
        esrc = edge.source
        edge2 = esrc.get_targets('assign')[0]
        etgt = edge2.target

        assert edge == edge2
        assert target.mapentry == etgt
        assert target == etgt.visualization

    complete_connection()
    import nord.sigurd.managers.connectionmanager as cm
    assert cm.pendingSource is None
    assert cm.pendingCtype is None


def test_cancel_conn():
    from mock import Mock, patch

    make_model_mock()

    @patch('builtins.render', Mock(), create=True)
    @patch('builtins.render2d', Mock(), create=True)
    @patch('builtins.camera', Mock(), create=True)
    def start_connection():
        source = vStatic('1', Mock())
        ctype = 'assign'
        em.proc_line("test_start_conn:nord.sigurd.managers.connectionmanager.start_conn")
        em.test_start_conn(event.EventInstance("", source, ctype))

    start_connection()
    import nord.sigurd.managers.connectionmanager as cm
    assert cm.pendingSource is not None
    assert cm.pendingCtype is not None

    @patch('builtins.render', Mock(), create=True)
    @patch('builtins.render2d', Mock(), create=True)
    @patch('builtins.camera', Mock(), create=True)
    def cancel_connection():
        em.proc_line("test_cancel_conn:nord.sigurd.managers.connectionmanager.cancel_conn")
        em.test_cancel_conn()

    cancel_connection()
    import nord.sigurd.managers.connectionmanager as cm
    assert cm.pendingSource is None
    assert cm.pendingCtype is None


@pytest.mark.skip(reason="INCOMPLETE TEST")
def test_draw():
    from mock import Mock, patch

    make_model_mock()

    @patch('builtins.render', Mock(), create=True)
    @patch('builtins.render2d', Mock(), create=True)
    @patch('builtins.camera', Mock(), create=True)
    def draw_connection():

        source = vStatic('1', Mock())
        ctype = 'assign'
        target = vData('2', Mock())
        import nord.sigurd.managers.connectionmanager as cm
        cm.draw(source, target, ctype)

    draw_connection()
    import nord.sigurd.managers.connectionmanager as cm
    assert cm.pendingSource is None
    assert cm.pendingCtype is None


@pytest.mark.skip(reason="remove method not implemented in code.")
def test_remove():
    # NOT IMPLEMENTED IN CODE YET.
    pass


@pytest.mark.skip(reason="INCOMPLETE TEST")
def test_getattr():
    import nord.sigurd.managers.connectionmanager as cm
    try:
        cm.ugly_banana(1)
        assert 1 == 0
    except AttributeError as e:
        assert str(e) == "No connection manager attribute named: ugly_banana"

    from mock import Mock, patch

    make_model_mock()

    @patch('builtins.render', Mock(), create=True)
    @patch('builtins.render2d', Mock(), create=True)
    @patch('builtins.camera', Mock(), create=True)
    def draw_connection():

        source = vStatic('1', Mock())
        target = vData('2', Mock())

        edge = Edge()
        edge.source = source.mapentry
        edge.target = target.mapentry

        import nord.sigurd.managers.connectionmanager as cm
        cm.draw_assign(edge)

        edge = target.mapentry.get_sources('assign')[0]
        esrc = edge.source
        edge2 = esrc.get_targets('assign')[0]
        etgt = edge2.target

        assert edge == edge2
        assert target.mapentry == etgt
        assert target == etgt.visualization

    draw_connection()
    import nord.sigurd.managers.connectionmanager as cm
    assert cm.pendingSource is None
    assert cm.pendingCtype is None
