"""
Copyright 2020 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
"""
import pytest
from panda3d.core import Point3
import nord.sigurd.shell.vismap as vm
import nord.sigurd.managers.objectmanager as om
from nord.wrigan.visibles.utils import dim_map, undim_map


@pytest.mark.panda3d()
def test_add_data_node(init_panda3d):
    # app = init_panda3d
    vm.create_node("Data", Point3(0.0, 0.0, 0.0), space="nord")
    karte = vm.get_working_map()
    # app.one_loop_iteration()
    r = render  # noqa: F821
    found = False
    newNode = karte.nodes[list(karte.nodes.keys())[0]]
    for v in list(r.get_children())[-15:]:
        if newNode.id == v.name:
            found = True
            break
    assert found
    dim_map()
    undim_map()
    # while True:
    #     app.one_loop_iteration()


@pytest.mark.panda3d()
def test_connect_data_to_static_node(init_panda3d):
    # app = init_panda3d
    vm.create_node("Data", Point3(-1.0, 0.0, 0.0), space="nord")
    karte = vm.get_working_map()
    data = karte.nodes[list(karte.nodes.keys())[0]]
    vm.create_node("Static", Point3(1.0, 0.0, 0.0), space="nord")
    static = karte.nodes[list(karte.nodes.keys())[1]]
    vm.create_edge(static, data, 'assign', None, None)
    # app.one_loop_iteration()
    r = render  # noqa: F821
    found = False
    latest_children = list(r.get_children())
    edge = karte.edges[list(karte.edges.keys())[0]]
    for v in latest_children:
        if v == edge.visualization.cEdge.data:
            found = True
            break
    assert found
    # while True:
    #     app.one_loop_iteration()


@pytest.mark.panda3d()
def test_add_handle_changes(init_panda3d):
    # app = init_panda3d
    karte = vm.get_working_map()
    karte.connect_cdc_loopback()
    vm.create_node("Dataobject", Point3(0.0, 0.0, 0.0), space="sushrut")
    # app.one_loop_iteration()
    r = render  # noqa: F821
    found = False
    karte.ship_changes()
    newNode = karte.nodes[list(karte.nodes.keys())[0]]
    vis = om.fetch(newNode.id)
    assert vis.mapentry._graph is newNode._graph
    for v in list(r.get_children()):
        if newNode.id == v.name:
            found = True
            break
    assert found

    changes = [
        {
            'change_type': 'change_node',
            'attribute': 'value',
            'change': {'type': 'Datatimestamp', 'value': '2020-09-13T00:00:00'},
            'delattr': False,
            'item_id': newNode.id,
            'prior': 'null',
            'timestamp': 1608601408.120389,
            'when': '2020-12-21T20:43:28.120389'
        },
        {
            'change_type': 'add_node',
            'entity': {
                'id': 'b86d9d96-1683-4aa2-...8853723b1',
                'name': 'Datafunction-Smart ...dic Sound',
                'space': 'sushrut',
                'type': 'datafunction'
                },
            'item_id': 'b86d9d96-1683-4aa2-...8853723b1',
            'timestamp': 1608601408.120448,
            'when': '2020-12-21T20:43:28.120448'
        },
        {
            'change_type': 'add_edge',
            'entity': {
                'from_space': 'sushrut',
                'rule': 'contains',
                'source': newNode.id + ':...me object',
                'space': 'nord',
                'target': 'b86d9d96-1683-4aa2-...8853723b1:...dic Sound',
                'to_space': 'sushrut'
            },
            'item_id': [newNode.id, 'b86d9d96-1683-4aa2-...8853723b1', 'contains'],
            'timestamp': 1608601408.120481,
            'when': '2020-12-21T20:43:28.120481'
        }
    ]
    karte.apply_changes(changes)
    dim_map()
    undim_map()
    # while True:
    #     app.one_loop_iteration()
