"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
Provide node class which provides static floats.
"""

from ._static import Static
import sys


class Staticbool(Static):
    """Facilitate static floats."""

    def __init__(self, graph, name="Static Bool"):
        super().__init__(graph, name, bool)

    def get(s):
        """
        Get the value from this data node
        """
        val = s.get_property('value')
        if type(val) == str:
            if val.lower() in ('0', 'f', 'no', 'n', 'false', 'zero'):
                rv = False
            else:
                rv = True
        elif val is None:
            rv = None
        elif val in (0, 0.0):
            rv = False
        else:
            rv = True
        return rv


sys.modules[__name__] = Staticbool
