"""
Copyright 2020 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

import nord.nord.nodes.Data as _Data
import sys


class Data(_Data):
    # def __init__(self, graph):
    #     super().__init__(graph)
    #     self.id = ng.id()
    #     self.type = self.__class__.__name__.lower()

    def to_dict(self, include_contents=False):
        """
        Since any of the Sushrut types can be instantiated via the backend
        the type needs to be populated, such that a BE shipped to FE
        dict version of the item includes the type.
        """
        rv = super().to_dict(include_contents)
        rv["type"] = self.__class__.__name__.lower()
        return rv

    def set_output_for_fe(self, runtime):
        runtime.set_output(self.id, self.to_dict(include_contents=True))

    def update_from_output(self, output):
        """
        An overridable method which, generically, looks for a value
        in the output dict, and applies that (with any casting needed)
        to this objects value. It will also update the ID of this instance.....(is this dangerous?)
        """
        if "value" in output and hasattr(self, '_type'):
            self.value = self._type(output['value'])
        elif "value" in output:
            self.set(output["value"])
        else:
            """
            It seems there is not a value to use, so, do nothing?
            """
            pass


sys.modules[__name__] = Data
