"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--
Enable assignment to so named nodes.
"""

import nord.nord.rules.DataAssign as DataAssign
from nord.sushrut.utils import get_modname_for_type
from nord.sigurd.utils.log import warn
import nord.nord.nodes.Data as Data
import sys
from nord.nord.exceptions import RuntimeException as RuntimeException


class DataarrayContains(DataAssign):
    """AssignDatabin sets the value of the Databin Node."""

    def apply(self):
        """Examine the target. Determine if it indicates the
        name of the attribute of self.value it wants, and
        validate it is of the correct type. If so, set the cargo
        of the edge to the "requested" type."""
        # TODO: perform the needed type checking here.
        if not self.edge.get_source().has_executed() or self.edge.get_source().get_unfollowed_sources():
            return False

        tgt = self.edge.target
        parent_value = self.edge.get_source().get()
        classname = tgt.__class__.__name__
        if not hasattr(tgt, 'name') and not hasattr(tgt, '_sushrut_dataobject_keyname'):
            raise RuntimeException(_("Contained Dataarray member missing 'name' attribute."))
        if not hasattr(tgt, '_sushrut_dataobject_keyname'):
            attrname = tgt.name
        else:
            attrname = tgt._sushrut_dataobject_keyname
        if not hasattr(parent_value, attrname):
            # There is no so named field in the parent from
            # which to grab the value, so we don't ignore that....
            # raise RuntimeException(_("No attribute of dataobject named {} in dataobject payload").format(attrname))
            # But, if we wanted to ignore it, we could do this instead...
            warn(_("Ignoring missing attribute of dataarray named {} in payload, target lacks value").format(attrname))
            return False
        value = getattr(parent_value, attrname)
        modname = get_modname_for_type(type(value))
        if modname == classname or isinstance(tgt, Data):
            self.edge.set_cargo(value)
        else:
            raise RuntimeException(_("Attribute of dataarray named {} incompatible type.").format(attrname))
        return True


sys.modules[__name__] = DataarrayContains
