"""
Copyright 2021 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

from nord.sigurd.visibles.static import Static as _Static
from nord.sigurd.utils.verboser import funlog
import nord.sigurd.managers.event as event


class Static(_Static):
    @funlog
    def update_tooltip(self, tooltip):
        if hasattr(self, 'mapentry'):
            val = self.mapentry.get()
            if val != self.mapentry.name:
                tooltip.set_text(f"{self.mapentry.name}: " + f"''{val}''")  # noqa: F821
            else:
                tooltip.set_text(self.mapentry.name)
            return
        tooltip.set_text(f"{self.name}: " + _("** Unset **"))  # noqa: F821

    @event.callback
    @funlog
    def applyLabelChange(self, newText):
        self.name = newText
        # Here we'll apply the name from the UI over the
        # generated name to the Map name, if there is one
        if hasattr(self, 'mapentry'):
            if hasattr(self.mapentry, 'name'):
                self.mapentry.name = self.name
            if not self.mapentry.is_set_property('value'):
                try:
                    self.mapentry._type(newText)
                    self.mapentry.set_property("value", newText)
                except ValueError:
                    pass
