"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

import pathlib
import sys
import pytest
import json
import datetime
import nord.nord.Map as Map  # noqa: E402
from nord.nord import Runtime as RT
Runtime = RT.Runtime


@pytest.mark.skip(reason="DEPRECATED TEST")
def test_sanitize_name():
    sys.path.append(str(pathlib.Path().cwd()) + '/sushrut/src/sushrut')
    import nord.sushrut.utils as utils
    assert utils.sanitize_name('!@#$%^&*(){}abc123,.') == 'abc123'
    assert utils.sanitize_name('12abc3!') == 'v_12abc3'
    assert utils.sanitize_name('__abc123__') == '__abc123__'


@pytest.mark.skip(reason="INCOMPLETE TEST")
def test_definableobject():
    sys.path.append(str(pathlib.Path().cwd()) + '/sushrut/src/sushrut')
    sys.path.append(str(pathlib.Path().cwd()) + '/sushrut/src')
    import nord.sushrut.nodes.Dataobject as Do
    import nord.sushrut.utils as utils
    import nord.nord.Edge as Edge
    import nord.nord.nodes.Data as Data
    datao = Do()
    tgt1 = Data()
    tgt1.name = 'Larry the $$ Rich Man'
    edge = Edge()
    edge.set_nodes(datao, tgt1, 'contains')
    a = utils.DefinableObject(datao)
    a.Larry_the_Rich_Man = 199
    assert tgt1.value == 199


@pytest.mark.skip(reason="DEPRECATED TEST")
def test_definable_as_dict():
    sys.path.append(str(pathlib.Path().cwd()) + '/sushrut/src/sushrut')
    sys.path.append(str(pathlib.Path().cwd()) + '/sushrut/src')
    import nord.sushrut.nodes.Dataobject as Do
    import nord.sushrut.utils as utils
    import nord.nord.Edge as Edge
    import nord.nord.nodes.Data as Data
    datao = Do()
    tgt1 = Data()
    tgt1.name = 'Larry the $$ Rich Man'
    edge = Edge()
    edge.set_nodes(datao, tgt1, 'contains')
    a = utils.DefinableObject(datao)
    a['Larry_the_Rich_Man'] = 199
    assert a['Larry_the_Rich_Man'] == 199
    assert tgt1.value == 199
    for (k, v) in a:
        assert k == 'Larry_the_Rich_Man'
        assert v == 199
    b = a['Larry_the_Rich_Man']
    b = 204  # noqa: F841
    assert tgt1.value == 199


@pytest.mark.skip(reason="INCOMPLETE TEST")
def test_dataobject():
    sys.path.append(str(pathlib.Path().cwd()) + '/sushrut/src/sushrut')
    import nord.nord.Map as Map
    from nord.nord.Runtime import Runtime
    import json
    fname = "sushrut/test/test_Dataobject.n"
    karte = Map(json.load(open(fname, 'r')))
    karte.run(Runtime(), None)
    datao = karte.nodes['e44f53b6-7e27-410a-a010-be4cfbf77087']
    rv = datao.get()
    assert rv.BINARY_B == 'Goober'
    rv.BINARY_B = 'Cheese Plate'
    assert karte.nodes['fad9c206-b038-4368-ad13-551fa05e5862'].value == 'Cheese Plate'


def test_dataobject_set_value_from_dict():
    import nord.sushrut.nodes.Dataobject as Do
    import nord.sushrut.nodes.Dataarray as Dataarray
    datao = Do(Map(None))
    datao.set({"attr1": 1, "attr2": "string", "attr3": {"sa1": 1, "sa2": {"ssa1": "another value", "arr": [1, 2, 3]}}})
    # set_data = datao.get()
    assert len(datao.get_targets("contains")) == 3
    tgt = datao.get_targets("contains")[0].get_target()
    assert tgt.get() == datao.get()[tgt._sushrut_dataobject_keyname]
    tgt = datao.get_targets("contains")[1].get_target()
    assert tgt.get() == datao.get()[tgt._sushrut_dataobject_keyname]
    tgt = datao.get_targets("contains")[2].get_target()
    assert tgt.get() == datao.get()[tgt._sushrut_dataobject_keyname]
    datao.set({"attr1": 1, "attr3": {"sa1": 1, "sa2": {"ssa1": "another value", "arr": [1, 3]}}})
    assert len(datao.get_targets("contains")) == 2
    tgt = datao.get_targets("contains")[1].get_target().get_targets("contains")[1].get_target()\
        .get_targets("contains")[1].get_target()
    assert type(tgt) == Dataarray
    assert tgt.get() == [1, 3]

    x = {"val": 5}

    def gen_test_function(X):
        def test_function(y):
            X["val"] = y
        return test_function
    datao.set({"attr1": gen_test_function(x), "arr": [1, 3]})

    tgt = datao.get_targets("contains")[0].get_target()
    func = tgt.get()
    assert x["val"] == 5
    func(6)
    assert x["val"] == 6


def test_dataobject_assign_object():
    # Load test map which "imports" nord.nord.Node via shoshoni
    # Ensure that the output of the assigned data object
    # has the correct contained sushrut objects
    # The map imports the Node object
    # Then, calls the member method get_op_runtime
    # and assigns that runtime to the target
    # data object.
    # The resulting data object should have
    # the runtime's structure as Contained
    # sushrut nodes.
    try:
        m = json.load(open("maps/test_dataobject_assign_object.n", "r"))
    except FileNotFoundError:
        m = json.load(open("sushrut/test/maps/test_dataobject_assign_object.n", "r"))
    karte = Map(m)
    vessal = Runtime()

    runtime_node = karte.nodes['66de5566-3f18-4202-b3ee-001a77b51251']

    # This is a hack to ensure that get_op_runtime returns something to
    # to the dataobject
    runtime_node._current_runtime = vessal
    # BROKEN .... OR GOOD IDEA THAT DOESN'T WORK
    # The brand new node instance is created by the map
    # but, it is never executed, and as such, does not have
    # a runtime to return. So, it seems another method is needed
    # to be used, that would return something.... Maybe another
    # class than Node...?

    vessal.initialize(karte, verbose=True)
    vessal.begin()
    vessal.continue_exec()
    # now, we should be able to interrogate the runtime node
    pass


def test_datetime_nest_interrogate():
    try:
        m = json.load(open("maps/test_dataobject_nested.n", "r"))
    except FileNotFoundError:
        m = json.load(open("sushrut/test/maps/test_dataobject_nested.n", "r"))
    karte = Map(m)
    resultant_dobj = karte.nodes['8a5b877c-3139-4194-9c0d-714ba7a93b30']
    now_dobj = karte.nodes['95e41b9b-1cda-4c0b-9e0f-f8b596b617fb']
    vessal = Runtime()
    vessal.initialize(karte, verbose=True)
    vessal.begin()
    vessal.continue_exec()

    attrs_checked = 0
    # now, we should be able to interrogate the runtime node
    for edge in resultant_dobj.get_targets('contains'):
        tgt = edge.get_target()
        if hasattr(tgt, '_sushrut_dataobject_keyname') and\
                hasattr(now_dobj._sushrut_datafunction_returnvalue, tgt._sushrut_dataobject_keyname):
            src = getattr(now_dobj._sushrut_datafunction_returnvalue, tgt._sushrut_dataobject_keyname)
            assert tgt.value == src
            attrs_checked += 1
    assert vessal.anomaly is None
    assert attrs_checked > 0
    assert attrs_checked == len(resultant_dobj.get_targets('contains'))


def test_create_from_datetime():
    import datetime
    import nord.sushrut.nodes.Dataobject as Dataobject

    vessal = Runtime()
    vessal.initialize(Map({}))
    datao = Dataobject(Map(None))
    datao.flag_exe(vessal)
    inp_dt = datetime.datetime.now()
    datao.set(inp_dt)
    for edge in datao.get_targets('contains'):
        tgt = edge.get_target()
        assert hasattr(inp_dt, tgt._sushrut_dataobject_keyname)
        assert tgt.get() == getattr(inp_dt, tgt._sushrut_dataobject_keyname)


def test_emulate_db_flow():

    class Cursor():
        def __init__(self, conn):
            self.conn = conn

        def execute(self, query):
            self.query = query
            Driver.execute_ran = Driver.run_count
            Driver.run_count += 1

        def fetchall(self):
            Driver.fetchall_ran = Driver.run_count
            Driver.run_count += 1
            return [[1, 2, 3], [1, 3, 5], [2, 8, 4]]

        def dummy_cursor_func(self):
            pass

    class Connection():
        def __init__(self, dsn):
            self.dsn = dsn

        def cursor(self):
            Driver.cursor_ran = Driver.run_count
            Driver.run_count += 1
            return Cursor(self)

        def close(self):
            pass

    class Driver():
        run_count = 1
        execute_ran = 0
        fetchall_ran = 0
        cursor_ran = 0
        connect_ran = 0

        def __init__(self):
            pass

        @staticmethod
        def connect(dsn):
            Driver.connect_ran = Driver.run_count
            Driver.run_count += 1
            return Connection(dsn)

    graph = {
        "extensions": {
            "sushrut": {
                "locations": {
                    "code": [
                        "null://.",
                    ],
                    "files": [
                        "dir:///Users/nate/Desktop/Dev/NoRD/sushrut/src/nord/sushrut",
                        "pkg://",
                    ],
                    "path": [
                        "dir:///Users/nate/Desktop/Dev/NoRD/sushrut/src/nord/sushrut",
                        "pkg://",
                    ]
                }
            }
        },
        "map": {
            "nodes": [
                {
                    "id": "driverinit",
                    "name": "INIT DRIVER",
                    "position": [0.0, 0.0, 0.0],
                    "space": "nord",
                    "type": "static",
                    "value": "REPLACEME"
                },
                {
                    "id": "query",
                    "name": "SELECT 1, 2, 'abc' UNION SELECT3, 4, 'def' UNION SELECT 5, 6, 'ghi'",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "staticstr",
                    "value": "SELECT 1, 2, 'abc' UNION SELECT3, 4, 'def' UNION SELECT 5, 6, 'ghi'"
                },
                {
                    "id": "dsn",
                    "name": "CONNECTION DETAILS'",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "staticstr",
                    "value": "DSN STRING"
                },
                {
                    "id": "driver",
                    "name": "Driver",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "dataobject",
                },
                {
                    "id": "connect",
                    "name": "connect",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "datafunction",
                },
                {
                    "id": "connection-obj",
                    "name": "Connection Obj",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "dataobject",
                },
                {
                    "id": "cursor",
                    "name": "cursor",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "datafunction",
                },
                {
                    "id": "cursor-obj",
                    "name": "Cursor Obj",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "dataobject",
                },
                {
                    "id": "execute",
                    "name": "execute",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "datafunction",
                },
                {
                    "id": "fetchall",
                    "name": "fetchall",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "datafunction",
                },
                {
                    "id": "result-arr",
                    "name": "result-arr",
                    "position": [0.0, 0.0, 0.0],
                    "space": "sushrut",
                    "type": "dataarray",
                },
            ],
            "edges": [
                {
                    "from_space": "nord",
                    "rule": "assign",
                    "source": "driverinit:driverinit",
                    "space": "nord",
                    "target": "driver:driver",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "contains",
                    "source": "driver:driver",
                    "space": "nord",
                    "target": "connect:connect",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "pass",
                    "source": "dsn:dsn",
                    "space": "nord",
                    "target": "connect:connect",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "assign",
                    "source": "connect:connect",
                    "space": "nord",
                    "target": "connection-obj:connection-obj",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "contains",
                    "source": "connection-obj:connection-obj",
                    "space": "nord",
                    "target": "cursor:cursor",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "flow",
                    "source": "connection-obj:connection-obj",
                    "space": "nord",
                    "target": "cursor:cursor",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "assign",
                    "source": "cursor:cursor",
                    "space": "nord",
                    "target": "cursor-obj:cursor-obj",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "contains",
                    "source": "cursor-obj:cursor-obj",
                    "space": "nord",
                    "target": "execute:execute",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "pass",
                    "source": "query:query",
                    "space": "nord",
                    "target": "execute:execute",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "contains",
                    "source": "cursor-obj:cursor-obj",
                    "space": "nord",
                    "target": "fetchall:fetchall",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "flow",
                    "source": "execute:execute",
                    "space": "nord",
                    "target": "fetchall:fetchall",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "assign",
                    "source": "fetchall:fetchall",
                    "space": "nord",
                    "target": "result-arr:result-arr",
                    "to_space": "sushrut"
                },
                {
                    "from_space": "sushrut",
                    "rule": "flow",
                    "source": "driver:driver",
                    "space": "nord",
                    "target": "connect:connect",
                    "to_space": "sushrut"
                },

            ],
        },
        "name": "pretend PG"
    }
    karte = Map(graph)
    # cursor = karte.nodes['cursor']
    driverinit = karte.nodes['driverinit']
    # driver = karte.nodes['driver']
    # connect = karte.nodes['connect']
    # execute = karte.nodes['execute']
    # fetchall = karte.nodes['fetchall']
    resultarr = karte.nodes['result-arr']

    driverinit.value = Driver()

    vessal = Runtime()
    vessal.initialize(karte, verbose=True)
    vessal.begin()
    vessal.continue_exec()
    assert vessal.anomaly is None
    assert vessal.input_wanters() == []
    assert resultarr.value == [[1, 2, 3], [1, 3, 5], [2, 8, 4]]
    assert Driver.connect_ran == 1
    assert Driver.cursor_ran == 2
    assert Driver.execute_ran == 3
    assert Driver.fetchall_ran == 4


def test_call_post_populate():
    try:
        m = json.load(open("maps/test_dataobject_nested.n", "r"))
    except FileNotFoundError:
        m = json.load(open("sushrut/test/maps/test_dataobject_nested.n", "r"))
    karte = Map(m)
    resultant_dobj = karte.nodes['8a5b877c-3139-4194-9c0d-714ba7a93b30']
    interm_dobj = karte.nodes['2d250c51-8717-4553-a385-1d4ec12c66f3']
    now_obj = karte.nodes['95e41b9b-1cda-4c0b-9e0f-f8b596b617fb']
    assert len(interm_dobj.get_targets("contains")) == 1
    assert len(resultant_dobj.get_targets("contains")) == 0
    vessal = Runtime()
    vessal.initialize(karte, verbose=True)
    vessal.begin()
    vessal.continue_exec()
    pre_change_children = len(resultant_dobj.get_targets("contains"))
    pre_cc_interm = len(interm_dobj.get_targets("contains"))
    karte.apply_changes(karte.get_changes())
    post_change_children = len(resultant_dobj.get_targets("contains"))
    post_cc_interm = len(interm_dobj.get_targets("contains"))
    vessal = Runtime()
    vessal.initialize(karte, verbose=True)
    vessal.begin()
    vessal.continue_exec()
    karte.apply_changes(karte.get_changes())
    post_run_children = len(resultant_dobj.get_targets("contains"))
    post_rc_interm = len(interm_dobj.get_targets("contains"))
    assert post_change_children == post_run_children
    assert post_cc_interm == post_rc_interm
    assert pre_cc_interm == post_rc_interm
    assert pre_change_children == post_run_children
    assert now_obj.value == datetime.datetime.now
