"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

import nord.nord.Edge as Edge
import sys


class Flow(Edge):
    """
    Flow is the connection between vertices that controls the execution of the
    map. At each step in the flow all the target's non-flow relationships are
    evaluated and potentially executed.

    >>> Flow() #doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
    Flow: [None]  --Flow-->  [None]
    """
    def follow(s, runtime, verbose=False):
        """
        Flow is a very generic edge. It requires no Rules since it just moves the runtime
        from one node to the next, regardless of the node subclass
        """
        if verbose:
            print("   ---> Flow following: {}".format(s))
        runtime.setNextNode(s.target, 'DOWN')


sys.modules[__name__] = Flow
