"""
Copyright 2019 NoRD Software Foundation

This file is part of NoRD.

NoRD is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

NoRD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with NoRD.  If not, see <https://www.gnu.org/licenses/>.

--

"""

import uuid
import nord.nord.Receiver as Receiver
import nord.nord.exceptions.RuntimeException as RuntimeException
import sys


class Return(Receiver):
    """
    Return nodes are the last nodes executed within a 
    container. They pass their value to their container's 
    assign and pass edges

    """
    def __init__(s, name=None, ntype=None):
        s.name = name
        if name is None:
            s.name = uuid.uuid4()
        s.type = ntype
        s.verbose = False

    def execute(s, runtime):
        """
            grab the assigned value and 
            set it to the cargo for any and all 
            pass and assign edges from the 
            container which holds this return

            A potential bug here is that the graph structure 
            allows the Return to be contained by more than 
            one container... So, throw a runtime exception
            if we detect more than one contains src
        """
        # Grab the runtime's verbosity so it can be used in hosOutEdges
        s.verbose = runtime.verbose

        containers = s.get_sources('contains')
        if len(containers) > 1:
            raise RuntimeException(_("Cannot share Return Receiver with multiple containers"))

        for container in [x.source for x in containers]:
            for passedge in container.get_targets('pass'):
                passedge.set_cargo(s.get_arg_values())

            for assignedge in container.get_targets('assign'):
                assignedge.set_cargo(s.get_arg_values())

        s.flag_exe(runtime)


    def has_out_edges(s):
        """
            over rides the check for out edges to follow any that exist
            , BUT, signal the runtime that it does not.
        """
        trules = (x for x in s.get_target_rules() if x not in ('flow'))
        for r in trules:
            for edge in s.get_targets(r):
                if not edge.been_followed():
                    edge.follow(s, s.verbose)
        return False


sys.modules[__name__] = Return